@@ -148,6 +148,8 @@ L</Swagger specification>, C<MyApp::authenticate_api_request>:
     );
   }
 
+This feature is EXPERIMENTAL and can change without notice.
+
 =cut
 
 use Mojo::Base 'Mojolicious::Plugin';
@@ -255,26 +257,53 @@ sub render_swagger {
 This method is called when this plugin is registered in the L<Mojolicious>
 application.
 
+C<%config> can contain these parameters:
+
+=over 4
+
+=item * route
+
+Need to hold a Mojolicious route object. See L</Protected API> for an example.
+
+This parameter is optional.
+
+=item * swagger
+
+A C<Swagger2> object. This can be useful if you want to keep use the
+specification to other things in your application. Example:
+
+  use Swagger2;
+  my $swagger = Swagger2->new->load($url);
+  plugin Swagger2 => {swagger => $swagger2};
+  app->defaults(swagger_spec => $swagger->tree);
+
+Either this parameter or C<url> need to be present.
+
+=item * url
+
+This will be used to construct a new L<Swagger2> object. The C<url>
+can be anything that L<Swagger2/load> can handle.
+
+Either this parameter or C<swagger> need to be present.
+
+=back
+
 =cut
 
 sub register {
   my ($self, $app, $config) = @_;
   my ($paths, $r, $swagger);
 
-  if ($config->{controller}) {
-    warn "Default 'controller' in Swagger2 plugin config is deprecated!";
-  }
+  $swagger = $config->{swagger} || Swagger2->new->load($config->{url} || '"url" is missing');
+  $swagger = $swagger->expand;
+  $paths   = $swagger->tree->get('/paths') || {};
 
-  $self->url($config->{url} || die "'url' is required config parameter");
-  $self->{controller} = $config->{controller};    # back compat
-  $app->helper(render_swagger => \&render_swagger);
-
-  $swagger = Swagger2->new->load($self->url)->expand;
-  $paths = $swagger->tree->get('/paths') || {};
+  $self->url($swagger->url);
+  $app->helper(render_swagger => \&render_swagger) unless $app->renderer->get_helper('render_swagger');
 
   $r = $config->{route};
   $r = $r->any($swagger->base_url->path->to_string) if $r and !$r->pattern->unparsed;
-  $r ||= $app->routes->any($swagger->base_url->path->to_string);
+  $r = $app->routes->any($swagger->base_url->path->to_string) unless $r;
 
   for my $path (keys %$paths) {
     for my $http_method (keys %{$paths->{$path}}) {
@@ -298,7 +327,7 @@ sub register {
 
 sub _generate_request_handler {
   my ($self, $method, $config) = @_;
-  my $controller = $config->{'x-mojo-controller'} || $self->{controller};    # back compat
+  my $controller = $config->{'x-mojo-controller'} or _die($config, "x-mojo-controller is missing in the swagger spec");
   my $defaults = {};
   my $handler;
 
@@ -389,6 +418,11 @@ sub _validate_input {
       $value += 0 if $type =~ /^(?:integer|number)/ and $value =~ /^-?\d/;
       $value = ($value eq 'false' or !$value) ? Mojo::JSON->false : Mojo::JSON->true if $type eq 'boolean';
 
+      # ugly hack
+      if (ref $p->{items} eq 'HASH' and $p->{items}{collectionFormat}) {
+        $value = Swagger2::SchemaValidator->_coerce_by_collection_format($p->{items}{collectionFormat}, $value);
+      }
+
       if ($in eq 'body') {
         warn "[Swagger2] Validate $in @{[$c->req->body]}\n" if DEBUG;
         push @e,
@@ -413,6 +447,10 @@ sub _validate_input {
   return \%v, \%input;
 }
 
+sub _die {
+  die "$_[1]: ", Mojo::Util::dumper($_[0]);
+}
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright (C) 2014, Jan Henning Thorsen
